Package: DeductiveR
Type: Package
Title: Deductive Rational Method
Version: 2.0.0
Date: 2026-05-08
Authors@R: person("Alonso", "Arriagada", email = "alonso.arriagada@usach.cl",
                  role = c("aut", "cre"))
Description: Apply the Deductive Rational Method to a monthly series of flow or precipitation data to fill in missing data. The method is as described in: Campos, D.F., (1984, ISBN:9686194444). 
License: GPL-3
LazyData: true
Encoding: UTF-8
Imports: dplyr, tidyr, ggplot2, moments, ggpubr, lmomco
Suggests: rmarkdown, knitr
VignetteBuilder: knitr
Depends: R (>= 2.10)
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-19 21:20:31 UTC; alobo
Author: Alonso Arriagada [aut, cre]
Maintainer: Alonso Arriagada <alonso.arriagada@usach.cl>
Repository: CRAN
Date/Publication: 2026-05-19 21:30:19 UTC
Built: R 4.5.2; ; 2026-05-19 23:58:40 UTC; unix

# Generated by roxygen2: do not edit by hand

export(DR)
export(DR_SVC)
export(DR_boxplot)
export(DR_plot)
export(DR_stats)
export(DR_timeseries_plot)
import(dplyr)
import(ggplot2)
import(ggpubr)
import(lmomco)
import(moments)
import(tidyr)

# DeductiveR
Deductive Rational Method for fill in missing data of flow or precipitation monthly time series.

# Requirements
Dependencies:
  dplyr, tidyr, ggplot2, moments, ggpubr, lmomco

# Installation
You can install the currently-released version of **DeductiveR** from CRAN with this R command:
```
install.packages("DeductiveR")
```
Alternatively, you can install the development version from GitHub with:
```
# install.packages("remotes")
remotes::install_github("Alobondo/DeductiveR")
library(DeductiveR)
```

# Usage
Functions | Description |
--- | --- |
```DR()``` | Apply the Deductive Rational Method to a monthly series of flow or precipitation data to fill in missing data. |
```DR_plot()``` | Plot the result from the application of the Deductive Rational Method. |
```DR_boxplot()``` | BoxPlot from the result from the application of the Deductive Rational Method. |
```DR_stats()``` | Statistics for the result from the application of the Deductive Rational Method. |
```DR_SVC()``` | Seasonal Variation Curves considering results from the application of the Deductive Rational Method. |
```DeductiveR::data``` | Load the example data. |

```DR()``` works with a dataframe with columns: date - year - month - day - station number (type date the first column and numeric the rest). This follows the same format as CAMELS-CL (```https://camels.cr2.cl/```) monthly flow file.

# Reporting bugs
If you find an error in some function, want to report a typo in the documentation or submit a recommendation, you can do it [here](https://github.com/Alobondo/DeductiveR/issues)

# Keywords
Hydrology, R package, Time Series

#' BoxPlot from the result from the application of the Deductive Rational Method
#' @export
#' @param dr data frame from DR function
#' @import dplyr tidyr ggplot2

DR_boxplot <- function(dr){
  data_long_month <- dr %>% pivot_longer(cols = -year, names_to = "Month", values_to = "Value")
  data_long_month$year <- as.numeric(data_long_month$year)

  aux <- as.data.frame(cbind(month.abb[seq(1,12)],colMeans(dr[2:13])))
  colnames(aux) <- c("Month","Mean")
  aux$Mean <- as.numeric(aux$Mean)
  # use factor for month to ensure Jan is the first and Dec the 12th
  aux$Month <- factor(aux$Month, levels = month.abb[seq(1,12)])
  data_long_month$Month <- factor(data_long_month$Month, levels = month.abb[seq(1,12)])

  ggplot(data_long_month, aes(x=as.factor(data_long_month$Month), y=data_long_month$Value)) +
    geom_boxplot(outlier.size = 1, outlier.alpha = 0.2, fill = "green") +
    scale_x_discrete(labels=month.abb[seq(1,12)], guide = guide_axis(angle = 90)) +
    geom_line(data = aux,aes(x=factor(aux$Month, levels = month.abb),y=aux$Mean, fill="mean"),size = 1.75, color = "red", linetype = "dashed", group =1,show.legend = TRUE) +
    scale_fill_manual("line", values=c(1),guide=guide_legend(override.aes = list(colour=c("red")))) +
    ggtitle("Monthly values filled with Deductive Rational Method") +
    xlab("Month") + ylab("Value") +
    theme_bw()

}

#' Seasonal Variation Curves considering results from the application of the Deductive Rational Method
#' @export
#' @param dr data frame from DR function
#' @import dplyr tidyr ggplot2 lmomco ggpubr

DR_SVC <- function(dr){
  Ordenados <- as.data.frame(apply(dr[,2:13], 2, sort, decreasing=TRUE))
  Ordenados <- as.data.frame(cbind(Exd_Probb = pp(Ordenados[,1]), Ordenados))

  CVE_probb <- data.frame(Exd_Probb = c(0.05, 0.25, 0.50, 0.75, 0.85, 0.90, 0.95))

  for (i in 2:13) {
    aux3 <- stats::approx(x = Ordenados$Exd_Probb, y = Ordenados[,i], xout = CVE_probb$Exd_Probb,
                   method = "linear", n = 1)
    CVE_probb <- cbind(CVE_probb,aux3$y)
  }

  colnames(CVE_probb)[2:13] <- colnames(Ordenados)[2:13]

  data_long_CVE_probb <- CVE_probb %>% pivot_longer(cols = -Exd_Probb, names_to = "variable", values_to = "value")

  data_long_CVE_probb <- data_long_CVE_probb[order(data_long_CVE_probb$Exd_Probb), ]

  plot1 <- ggplot(data = data_long_CVE_probb, aes(x = data_long_CVE_probb$variable, y = data_long_CVE_probb$value, color = factor(data_long_CVE_probb$Exd_Probb*100))) +
    geom_point(shape = 1, size = 2, fill = "white") +
    geom_line(aes(group = data_long_CVE_probb$Exd_Probb),size = 0.85) +
    scale_x_discrete(labels=month.abb[seq(1,12)], guide = guide_axis(angle = 90))+
    ggtitle("Seasonal Variation Curves for monthly values \nfilled with Rational Deductive Method") +
    labs(color = "Exd. Probb. [%]") + xlab("Month") + ylab("Value") +
    theme_bw()

  tbl <- ggtexttable(cbind('Exd. Probb [%]' = CVE_probb[,1]*100, round(CVE_probb[,2:13],3)),rows=NULL)

  ggarrange(plot1, tbl,
            ncol = 1, nrow = 2,
            heights = c(1.8,1))

}

#' Plot the result from the application of the Deductive Rational Method
#' @export
#' @param dr data frame from DR function
#' @param facet logical, if TRUE, the plot will be faceted by year
#' @import dplyr tidyr ggplot2

DR_plot <- function(dr, facet = FALSE){
  data_long_month <- dr %>% pivot_longer(cols = -year, names_to = "Month", values_to = "Value")
  data_long_month$year <- as.numeric(data_long_month$year)

  aux <- as.data.frame(cbind(month.abb[seq(1,12)],colMeans(dr[2:13])))
  colnames(aux) <- c("Month","Mean")
  aux$Mean <- as.numeric(aux$Mean)
  # use factor for month to ensure Jan is the first and Dec the 12th
  aux$Month <- factor(aux$Month, levels = month.abb[seq(1,12)])
  data_long_month$Month <- factor(data_long_month$Month, levels = month.abb[seq(1,12)])

  if (facet) {
    ggplot(data_long_month, aes(x = factor(Month, levels = month.abb),
                                y = Value,
                                group = year,
                                color = factor(year))) +
      geom_line(size = 1, alpha = 0.75) +
      facet_wrap(~ year) +
      # add red dashed line for monthly mean
      geom_line(data = aux,
                aes(x = factor(Month, levels = month.abb),
                    y = Mean,
                    group = 1,
                    color = "mean"),
                size = .75, linetype = "dashed", show.legend = TRUE) +
      scale_color_manual(values = c("mean" = "red"),
                         breaks = c(levels(factor(data_long_month$year)), "mean"),
                         labels = c(levels(factor(data_long_month$year)), "mean")) +
      labs(color = "year") +
      ggtitle("Monthly values filled with Deductive Rational Method") +
      xlab("Month") + ylab("Value") +
      theme_bw()+
      theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1))

  } else {

    ggplot(data_long_month, aes(x=as.factor(data_long_month$Month), y=data_long_month$Value, group=data_long_month$year, color=data_long_month$year)) +
      labs(color='year') +
      geom_line(size = 1, alpha = 0.75) +
      scale_x_discrete(labels=month.abb[seq(1,12)], guide = guide_axis(angle = 90)) +
      geom_line(data = aux,aes(x=factor(aux$Month, levels = month.abb),y=aux$Mean,fill="mean"),size = 1.75, color = "red", linetype = "dashed", group =1,show.legend = TRUE) +
      scale_fill_manual("line", values=c(1),guide=guide_legend(override.aes = list(colour=c("red")))) +
      ggtitle("Monthly values filled with Deductive Rational Method") +
      xlab("Month") + ylab("Value") +
      theme_bw()

  }


}

#' Statistics for the result from the application of the Deductive Rational Method
#' @export
#' @param dr data frame from DR function
#' @import dplyr tidyr moments

DR_stats <- function(dr){
  data_long_month <- dr %>% pivot_longer(cols = -year, names_to = "Month", values_to = "Value")

  # order the result by month from Jan to Dec
  data_long_month %>% group_by(Month) %>%
    summarise(
      N = n(),
      Max = max(Value, na.rm = TRUE),
      Min = min(Value, na.rm = TRUE),
      Mean = mean(Value, na.rm = TRUE),
      SD = stats::sd(Value, na.rm = TRUE),
      Median = stats::median(Value, na.rm = TRUE),
      CV = stats::sd(Value, na.rm = TRUE) / mean(Value, na.rm = TRUE),
      Skew = skewness(Value, na.rm = TRUE),
      Kurtosis = kurtosis(Value, na.rm = TRUE),
      IQR = stats::IQR(Value, na.rm = TRUE),
      Qs_2 = stats::quantile(Value, 0.25, na.rm = TRUE),
      Qs_4 = stats::quantile(Value, 0.75, na.rm = TRUE)
    ) %>%
    mutate(Month = factor(Month, levels = month.abb)) %>%
    arrange(Month)

}

#' Plot raw and infilled monthly time series
#' @export
#' @param raw_data data frame with columns: date - year - month - day - station (Type date and numeric the rest). Assumes station data is in the 5th column.
#' @param infilled_data data frame from DR function (wide format: year, month1, month2, ...).
#' @import dplyr tidyr ggplot2
#'
DR_timeseries_plot <- function(raw_data, infilled_data){

  # Ensure 'date' column in raw_data is Date type
  raw_data[[1]] <- as.Date(raw_data[[1]])

  # Prepare original data (assuming station is the 5th column)
  df_original_plot <- raw_data %>%
    select(date = 1, Value = 5) %>%
    mutate(Type = "Original")

  # Prepare infilled data
  df_infilled_plot <- infilled_data %>%
    pivot_longer(cols = -year, names_to = "Month_abb", values_to = "Value") %>%
    mutate(
      Month_num = match(Month_abb, month.abb),
      date = as.Date(paste(year, Month_num, "01", sep = "-"), format = "%Y-%m-%d"),
      Type = "Infilled"
    ) %>%
    select(date, Value, Type)

  # Combine data
  df_combined <- bind_rows(df_original_plot, df_infilled_plot)

  # Plotting
  ggplot(df_combined, aes(x = date, y = Value, color = Type, linetype = Type)) +
    geom_line(linewidth = 1.5) +
    labs(
      title = "Raw vs. Infilled Monthly Time Series",
      x = "Date",
      y = "Value"
    ) +
    theme_bw() +
    scale_color_manual(values = c("Original" = "blue", "Infilled" = "red"))
}

#' Monthly flow data from Estero Culebron DGA station
#'
#' A dataset containing dates and monthly flow values.
#'
#' @format A data frame with 480 rows and 5 variables:
#' \describe{
#'   \item{date}{dates}
#'   \item{year}{years}
#'   \item{month}{months}
#'   \item{day}{days}
#'   \item{4400001}{flows, in cubic meters per second}
#' }
#' @source \url{https://camels.cr2.cl/}
"data"

## Global variables for R CMD check

# Define global variables to avoid "no visible binding for global variable" notes during R CMD check
# vARIABLES: Exd_Probb Mean Month Month_abb Month_num Type Value day month station year

utils::globalVariables(c("Exd_Probb", "Mean", "Month", "Month_abb", "Month_num", "Type", "Value", "day", "month", "station", "year"))

#' Apply the Deductive Rational Method to a monthly series of flow or
#' precipitation data to fill in missing data.
#' @export
#' @param data data frame with columns: date - year - month - day - station (Type date and numeric the rest)
#' @param negative logical, if TRUE, negative values will be set to zero in the final result. Default is FALSE.
#' @import dplyr tidyr

DR <- function(data, negative = FALSE){
  # The method only allows you to fill in up to 11 months for incomplete years
  # and requires a record with at least 10 full years.

  # A matrix of years x months is created
  colnames(data) <- c("date", "year", "month", "day", "station")
  data_wide <- data %>%
    select(-day, -date) %>%
    pivot_wider(names_from = month, values_from = station) %>%
    mutate(across(everything(), as.numeric))

  # A long format DF a is created
  data_long <- data_wide %>% pivot_longer(cols = -year, names_to = "month", values_to = "Station")

  # Count missing months by chronological years
  meses_faltantes <- stats::aggregate(Station ~ year, data=data_long, function(x) {sum(is.na(x))}, na.action = NULL)

  # Number of complete years
  complete_y <- sum(meses_faltantes$Station == 0, na.rm=TRUE)
  if(complete_y < 10){stop("There is less than 10 years with 12 monthly data")}

  # Number of incomplete years
  incomplete_y <- sum(meses_faltantes$Station != 0, na.rm=TRUE)

  # Number of years with 12 missing months
  incomplete_y_12 <- sum(meses_faltantes$Station > 11, na.rm=TRUE)

  # Years without data
  agnos_sin <- filter(meses_faltantes, meses_faltantes$Station > 11)

  # Years without data is eliminated
  if (length(agnos_sin$year) != 0) {
  data_wide <- data_wide %>% slice(-match(agnos_sin$year, data_wide$year))
  }

  # Sum of each year
  Suma <- data_wide %>% select(-1) %>% rowSums(na.rm=T)
  data_wide <- data_wide %>% mutate(Suma)

  #  Percentages matrix
  data_porc <- (select(data_wide, -1, -14) / Suma) * 100

  # Average percentages vector
  Porc_prom <- data_porc %>% summarise_if(is.numeric, mean,na.rm=T)

  # Auxiliary matrix
  Aux <- data_wide %>% select(-1, -14) %>% "*"(0)
  Aux[is.na(Aux)] <- 1

  # Auxiliary vectors with missing percentages
  V_aux <- as.matrix(Aux) %*% t(Porc_prom)
  V_aux_comp <- 100 - V_aux

  # Auxiliary vectors to generate fill values with DR method
  Rep_aux <- rep(V_aux_comp, each = 12)
  Rep_aux_2 <- rep(Suma, each = 12)
  Rep_aux_3 <- as.double(rep(Porc_prom, times = length(Rep_aux)/12))
  Rep_aux_4 <- as.vector(t(as.matrix(Aux)))

  # Values are calculated to fill in missing months
  DR <- ((Rep_aux_2 * Rep_aux_3) / Rep_aux)*Rep_aux_4

  # Final element with existing and filled data
  DR_final <- data_wide %>% select(-1, -14)
  DR_final[is.na(DR_final)] <- 0

  DR <- t(matrix(DR,nrow = 12))
  DR_final <- DR_final + DR

  # Avoid negative values if 'negative' parameter is TRUE
  if (negative) {
    DR_final[DR_final < 0] <- 0
  }

  DR_final <- cbind(data_wide$year, DR_final)
  colnames(DR_final) <- c("year",month.abb[seq(1,12)]) # Column names

  print(paste0("Number of complete years: ",complete_y))

  print(paste0("Number of incomplete years: ",incomplete_y))

  print(paste0("Number of years without data: ",incomplete_y_12))

  DR_final

}

---
title: "DeductiveR"
author: "Alonso Arriagadada M.  <alonso.arriagada@usach.cl>"
version: "1.1.0"
date: "29-11-2021"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{DeductiveR}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

DeductiveR is an R package for fill in missing data of flow or precipitation monthly time series using the Deductive Rational Method, proposed by Campos (Campos, D. F., Procesos del ciclo hidrologico, Universidad Autonoma de San Luis Potosi, 1992, Mexico).

## Requirements
Dependencies: dplyr, reshape2, ggplot2, moments, ggpubr, lmomco

## Installation
You can install the development version of DeductiveR from GitHub with:
```{r setup,eval = FALSE}
# install.packages("remotes")
remotes::install_github("Alobondo/DeductiveR")
library(DeductiveR)
```

## Usage
Functions | Description |
--- | --- |
```DR()``` | Apply the Deductive Rational Method to a monthly series of flow or precipitation data to fill in missing data. |
```DR_plot()``` | Plot the result from the application of the Deductive Rational Method. |
```DR_stats()``` | Statistics for the result from the application of the Deductive Rational Method. |
```DR_boxplot()``` | Monthly boxplot considering result from the application of the Deductive Rational Method. |
```DR_SVC()``` | Seasonal Variation Curves considering results from the application of the Deductive Rational Method. |

```DR()``` works with a dataframe with columns: date - year - month - day - station number (type date the first column and numeric the rest). This follows the same format as CAMELS-CL (```https://camels.cr2.cl/```) monthly flow file.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DR_plot_package.R
\name{DR_plot}
\alias{DR_plot}
\title{Plot the result from the application of the Deductive Rational Method}
\usage{
DR_plot(dr, facet = FALSE)
}
\arguments{
\item{dr}{data frame from DR function}

\item{facet}{logical, if TRUE, the plot will be faceted by year}
}
\description{
Plot the result from the application of the Deductive Rational Method
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{data}
\alias{data}
\title{Monthly flow data from Estero Culebron DGA station}
\format{
A data frame with 480 rows and 5 variables:
\describe{
  \item{date}{dates}
  \item{year}{years}
  \item{month}{months}
  \item{day}{days}
  \item{4400001}{flows, in cubic meters per second}
}
}
\source{
\url{https://camels.cr2.cl/}
}
\usage{
data
}
\description{
A dataset containing dates and monthly flow values.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DR_stats_package.R
\name{DR_stats}
\alias{DR_stats}
\title{Statistics for the result from the application of the Deductive Rational Method}
\usage{
DR_stats(dr)
}
\arguments{
\item{dr}{data frame from DR function}
}
\description{
Statistics for the result from the application of the Deductive Rational Method
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DR_timeseries_plot_package.R
\name{DR_timeseries_plot}
\alias{DR_timeseries_plot}
\title{Plot raw and infilled monthly time series}
\usage{
DR_timeseries_plot(raw_data, infilled_data)
}
\arguments{
\item{raw_data}{data frame with columns: date - year - month - day - station (Type date and numeric the rest). Assumes station data is in the 5th column.}

\item{infilled_data}{data frame from DR function (wide format: year, month1, month2, ...).}
}
\description{
Plot raw and infilled monthly time series
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DR_boxplot_package.R
\name{DR_boxplot}
\alias{DR_boxplot}
\title{BoxPlot from the result from the application of the Deductive Rational Method}
\usage{
DR_boxplot(dr)
}
\arguments{
\item{dr}{data frame from DR function}
}
\description{
BoxPlot from the result from the application of the Deductive Rational Method
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DR_SVC_package.R
\name{DR_SVC}
\alias{DR_SVC}
\title{Seasonal Variation Curves considering results from the application of the Deductive Rational Method}
\usage{
DR_SVC(dr)
}
\arguments{
\item{dr}{data frame from DR function}
}
\description{
Seasonal Variation Curves considering results from the application of the Deductive Rational Method
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DR_package.R
\name{DR}
\alias{DR}
\title{Apply the Deductive Rational Method to a monthly series of flow or
precipitation data to fill in missing data.}
\usage{
DR(data, negative = FALSE)
}
\arguments{
\item{data}{data frame with columns: date - year - month - day - station (Type date and numeric the rest)}

\item{negative}{logical, if TRUE, negative values will be set to zero in the final result. Default is FALSE.}
}
\description{
Apply the Deductive Rational Method to a monthly series of flow or
precipitation data to fill in missing data.
}
